;***************************************************************************
; 
; File Name		:'MEMORY.asm"
; Title			:
; Date			:
; Version		:
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Target MCU		:AT90S8515
;
; DESCRIPTION
;
; All routines that put data in or out of memory buffers
; 
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.07.29  dvh	Creation
;	0.02	98.08.28  dvh	Regularized to XL,XH notation instead of R26.R27 etc
;	0.03	98.09.01  dvh	STRING_SEROUT now enables UDRIE, so we start transmitting.
;				The UDRIE ISR will turn it off when we're done talking.
;	0.04	98.10.19  dvh	Fixed a stack bug that would crash you if the serout buffer became
;				full.
;	0.05	99.05.27  dvh   Constructing generalized buffer manager routines,
;				called by pointing to the desired buffer in Y, and
;				they all work the same. Note: LPM uses Z, so Z dosen't
;				work as a pointer to the destination.
;
;				New buffer structure:
;				LSBBBBBBBBBBBBBBBBBBBBBB
;				L is the length byte, and is located AT the 
;				buffer address. The buffer starts at address
;				+2, and runs to address + len + 2
;				If L = 0, then the buffer is empty.
;				S is the buffer size, if L=S then the buffer is 
;				full.
;
;				All FIFO buffers use a routine for "put", and another 
;				for "take". 
;
;	0.06	99.09.09  dvh	Added TEST_SPACE which is used when stuffing a string
;				into a buffer. It returns a zero if the destination
;				buffer is too full to accept the string.
;				The calling routine can then abort or whatever
;				based on that.
;
;***************************************************************************
;
;Each buffer looks like this:
;<S><L>DATA
;S is the current size of the buffer (in this case 4)
;L is the length of the buffer, which could be anything.
;If L-S=0 then the buffer is full.
;The length does not change during run-time, size is dependent on the 
;contents, can be anywhere from zero to equal to size.
;Initially, they are set to zero size, and whatever length is set up
;in EQUATES.INC. 
;
INIT_Buffers:
	
	cli				;Turn off all ints for a moment
	rcall	Clean_Ram		;Clean up the ram, and flag buffers
	rcall	Flag_Ram		;Note that this does NOT result in
					;buffers set to arbitrary size,
					;because of the following routines
					;that explicitly set the buffer
					;lengths and sizes.

	ldi	YH,high(Morse_Out_Buf)	;Get the buffer head
	ldi	YL,low(Morse_Out_Buf)	;
	ldi	TEMP,0			;I'm empty
	st	Y+,TEMP			;
	ldi	TEMP,(Morse_Size)	;and I can be this big
	st	Y,TEMP			;

	ldi	YH,high(Serial_In_Buf)	;Get the buffer head
	ldi	YL,low(Serial_In_Buf)	;
	ldi	TEMP,0			;
	st	Y+,TEMP			;
	ldi	TEMP,(Serin_Size)	;
	st	Y,TEMP			;
	
	ldi	YH,high(Serial_Out_Buf)	;Get the buffer head
	ldi	YL,low(Serial_Out_Buf)	;
	ldi	TEMP,0			;
	st	Y+,TEMP			;
	ldi	TEMP,(Serout_Size)	;
	st	Y,TEMP			;

	ldi	YH,high(LCD_Out_Buf)	;Get the buffer head
	ldi	YL,low(LCD_Out_Buf)	;
	ldi	TEMP,0			;
	st	Y+,TEMP			;
	ldi	TEMP,(LCD_out_Size)	;
	st	Y,TEMP			;

	ldi	YH,high(VFD_Out_Buf)	;Get the buffer head
	ldi	YL,low(VFD_Out_Buf)	;
	ldi	TEMP,0			;
	st	Y+,TEMP			;
	ldi	TEMP,(VFD_out_Size)	;
	st	Y,TEMP			;

	sei				;Let the interrupts fly!
	ret				;
;
;************************************************************
;
;Used for strings that must fit into the dest buffer.
;Z points to the string, and Y to the buffer.
;
Test_Space:
	push	ZH			;Save the original pointer
	push	ZL			;
	push	YH			;
	push	YL			;
	ldi	TEMP,0			;
Test_Space_Loop:
	lpm				;Get the byte into R0 and incr Z
	and	R0,R0			;Is it null
	breq	Test_Space_Check	;Yes, all done
	ld	R0,Z+			;inc pointer
	inc	TEMP			;inc byte counter
	rjmp	Test_Space_Loop		;Again.

Test_Space_Check:
	ld	TEMP2,Y+		;Get current length
	ld	TEMP3,Y			;and size
	sub	TEMP3,TEMP2		;Size - contents = freespace 
	cp	TEMP3,TEMP		;CP Freespace to string len
	brlo	Test_Space_Full		;

Test_Space_NotFull:
	ldi	TEMP,1			;
	and	TEMP,TEMP		;
	rjmp	Test_Space_Exit		;

Test_Space_Full:			;
	ldi	TEMP,0			;
	and	TEMP,TEMP		;
	;rjmp	Test_Space_Exit		;

Test_Space_Exit:
	pop	YL			;
	pop	YH			;
	pop	ZL			;
	pop	ZH			;
	ret				;
;
;************************************************************
;
;This routine takes a null terminated string, pointed to by Z, 
;and places it in the buffer pointed to by Y
;Will place as much of the string as will fit into the dest, and no more.
;The calling routine should have used Test_Space if it matters.
;
String_out:
	lpm			;Get the byte into R0 and incr Z
	and	R0,R0		;Is it null
	breq	STR_out_Exit	;Yes, all done
	mov	TEMP,R0		;
	rcall	Store_Buffer	;Take this byte and shove it
	ld	TEMP,Z+		;Inc Z, toss the data.
	rjmp	String_Out	;

Str_Out_Exit:
	ret
;
;
;************************************************************
;
;Stores a byte in a buffer pointed to by YH,YL
;Data is in TEMP
;Trashes TEMP2,TEMP3
;
Store_Buffer:

	;First check that the buffer can take more.
	push	YH		;
	push	YL		;
	ld	TEMP2,Y+	;Get the current length
	ld	TEMP3,Y+	;Get the current size
	cp	TEMP2,TEMP3	;Are they equal?
	breq	Store_Buffer_Done;Yes, then just exit, can't take more

Store_Buf_Data:
	add	YL,TEMP2		;Add the length
	brcc	Store_buf_Data_B	;
	inc	YH			;

Store_buf_Data_B:
	st	Y,TEMP			;Store the data
	inc	TEMP2			;

Store_Buffer_Done:
	pop	YL			;
	pop	YH			;
	st	Y,TEMP2			;store the length

	ret

;
;************************************************************
;
;This routine is called by the serial input ISR
;The byte to store is in TEMP
;This is a special case, only because of the Xoff handling.
;
Store_Serin:
	push	YL	;
	push	YH	;
	push	TEMP2	;
	push	TEMP3	;
	push	TEMP	;Because I want TEMP first

	ld	TEMP2,Y+	;Get the current length
	ld	TEMP3,Y+	;Get the current size
	cp	TEMP2,TEMP3	;Are they equal?
	breq	Store_Ser_in_Fail;Yes, then just exit, can't take more

	cpi	TEMP,(SERIN_SIZE-1)	;Is it almost full?
	breq	Store_Ser_in_Xoff	;Store, but say xoff

Store_Ser_In_Xoff:
	rcall	HS_XOFF			;I'm full

Store_Ser_Data:
	pop	TEMP			;
	ldi	YH,high(Serial_in_Buf)	;Get the buffer head
	ldi	YL,low(Serial_in_Buf)	;
	rcall	Store_Buffer		;

Store_Ser_in_Fail:
	pop	TEMP	;Balance the stack in case of fail

Store_Ser_in_Done:
	;Already popped TEMP above, either success or fail
	pop	TEMP3	;
	pop	TEMP2	;
	pop	YH	;
	pop	YL	;
	ret
;
;*************************************************************
;
;Used to delete a single char off the head of a buffer.
;Point YH,YL to beginning of buffer Head
;Returns junk in TEMP and TEMP2
;

Kill_Head:

	ld	TEMP2,Y		;Get current length
	cpi	TEMP2,0		;If no string, then
	breq	Kill_Head_Done	;there's nothing to do

	;YH,YL working pair pointing at head of the data
	push	TEMP2		;
	push	YH		;
	push	YL		;
	adiw	YL,2		;Point at the data
	mov	XH,YH		;
	mov	XL,YL		;
	adiw	XL,1		;Point X at next cell

Kill_Head_Loop:
	ld	TEMP,X+		;Shuffle down
	st	Y+,TEMP		;
	dec	TEMP2		;
	brne	Kill_Head_Loop	;

	pop	YL		;Get the original address
	pop	YH		;back into Y
	pop	TEMP2		;Get the original length
	dec	TEMP2		;-1
	st	Y,TEMP2		;store it

Kill_Head_Done:
	ret			;

;
;*************************************************************
;
;Used to delete a single char off the tail of a buffer
;Point YH,YL to beginning of buffer
;Returns junk in temp
;
Kill_Tail:
	ld	TEMP,Y		;Get the length
	cpi	TEMP,0		;Is it zero?
	breq	Kill_Tail_Done	;If so, we're done.
	dec	TEMP		;else make it shorter
	st	Y,TEMP		;and store it back.
Kill_Tail_Done:
	ret			;
;
;***************************************************************************
;
;Ram_Init:	;Load all variables with harmless or nonsense values
;	rcall	Clean_Ram		;Go nuke the ram space
;	rcall	FLAG_RAM		;
;	ret
;
;**********************************************************************************
;Debug only, stores distinctive data into each buffer, making it easy to see them
;in the simulator.
;
Flag_Ram:
;	

F_1:
	ldi	ZH,high(SERIAL_IN_BUF)	;
	ldi	ZL,low(SERIAL_IN_BUF)	;
	ldi	TEMP,$22		;
	ldi	TEMP2,SER_SIZE		;
	rcall	Fill_Buf		;

F_2:	
	ldi	ZH,high(Serial_OUT_BUF)	;
	ldi	ZL,low(Serial_OUT_BUF)	;
	ldi	TEMP,$33		;
	ldi	TEMP2,SER_SIZE		;
	rcall	Fill_Buf		;

F_3:	
	ldi	ZH,high(Morse_OUT_BUF)	;
	ldi	ZL,low(Morse_OUT_BUF)	;
	ldi	TEMP,$44		;
	ldi	TEMP2,Morse_SIZE	;
	rcall	Fill_Buf		;

F_4:	
	ldi	ZH,high(LCD_OUT_BUF)	;
	ldi	ZL,low(LCD_OUT_BUF)	;
	ldi	TEMP,$45		;
	ldi	TEMP2,LCD_OUT_SIZE	;
	rcall	Fill_Buf		;

F_5:	
	ldi	ZH,high(VFD_OUT_BUF)	;
	ldi	ZL,low(VFD_OUT_BUF)	;
	ldi	TEMP,$46		;
	ldi	TEMP2,VFD_OUT_SIZE	;
	rcall	Fill_Buf		;

F_Fin:	
	ret				;Done.

;
;Fill a given buffer to a given size with 
;given data.
;
Fill_Buf:
	st	Z+,TEMP			;Fill a byte
	dec	TEMP2			;One less to fill
	brne	Fill_Buf		;Are we done?
	ret				;yes, exit
;
;***************************************************************************
;
;This routine flushes any previous junk out of our workspace.
;Avoids any pattern-sensitivity or programmer headspace errors with
;uninitialized variables.
;
Clean_Ram:
	ldi	ZH,$00			;Point @ $0060
	ldi	ZL,$60			;
	clr	TEMP			;

CleanLoop:
	st	Z+,TEMP			;Store $00 at each location
	cpi	ZH,$02			;If not at least here, then go do it again
	brne	CleanLoop		;
	cpi	ZL,$5C			;DONT EAT THE STACK!
	brne	CleanLoop		;Getting close!
	ret				;Done.

;
;*************************************************************
;
;R28,29 are buffer tail pointer R30,31 are buffer head
;Return number of chars stored between R28,30
;Zero flag on exit says no more 
;
Check_It:
	ld	TEMP2,Y+	;Get the size
	ld	TEMP,Y		;Get the length
	sbc	TEMP,TEMP2	;If the result is zero,
	ret			;then no more will go.
;
;*************************************************************
;